package es.upv.server.services;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.mortbay.log.Log;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import es.upv.server.dao.GenericDao;
import es.upv.shared.dto.RespuestaPregunta;
import es.upv.shared.dto.TipoPregunta;
import es.upv.shared.services.PreguntasService;

@Service("preguntasService")
public class PreguntasServiceImpl implements PreguntasService {

	GenericDao<TipoPregunta, Long> tipoPreguntaDao;
	GenericDao<RespuestaPregunta, Long> respuestaDao;

	public GenericDao<TipoPregunta, Long> getTipoPreguntaDao() {
		return tipoPreguntaDao;
	}

	@Autowired
	public void setTipoPreguntaDao(GenericDao<TipoPregunta, Long> tipoPreguntaDao) {
		this.tipoPreguntaDao = tipoPreguntaDao;
	}
	
	public GenericDao<RespuestaPregunta, Long> getRespuestaDao() {
		return respuestaDao;
	}

	@Autowired
	public void setRespuestaDao(GenericDao<RespuestaPregunta, Long> respuestaDao) {
		this.respuestaDao = respuestaDao;
	}

	@Override
	public List<TipoPregunta> obtenerTiposPregunta() {
		return tipoPreguntaDao.getAll();
	}

	@Override
	public List<RespuestaPregunta> obtenerRespuestasPregunta(Long preguntaId) {
		Map<String, Object> queryParams = new HashMap<String, Object>();
		queryParams.put("pregunta", preguntaId);
		return respuestaDao.findByNamedQuery("obtenerRespuestasPregunta", queryParams);
	}

	@Override
	public void eliminaTipoPregunta(Long tipoPreguntaId) {
		tipoPreguntaDao.remove(tipoPreguntaId);
	}

	@Override
	public TipoPregunta actualizaTipoPregunta(TipoPregunta tipoPregunta) {
		return tipoPreguntaDao.save(tipoPregunta);
	}

	/* Para guardar las respuestas, primero se buscan las que existen en la base de datos
	 * Se crea un map para compararlas con las respuestas enviadas a guardar.
	 * 
	 * Para cada una de las respuestas a guardar, se busca si eset en la base de datos comparando el id (en el map)
	 * y si existe, se elimina del Map. Si no existe, se guarda como nueva
	 * 
	 * Al terminar el bucle, las respuestas que existan en el map, sern las que estabn en base de dastos pero
	 * han sido eliminadas
	 * 
	 * (non-Javadoc)
	 * @see es.upv.shared.services.PreguntasService#guardarRespuestasPregunta(java.lang.Long, java.util.List)
	 */
	@Override
	@Transactional(rollbackFor=Exception.class)
	public List<RespuestaPregunta> guardarRespuestasPregunta(Long preguntaId, List<RespuestaPregunta> respuestas) {
		// primero obtener las respuestas actuales, ver si es necesario eliminar alguna
		List<RespuestaPregunta> respuestasGuardadads = new ArrayList<RespuestaPregunta>();
		List<RespuestaPregunta> respuestasDB = obtenerRespuestasPregunta(preguntaId);
	
		Map<Long, RespuestaPregunta> respuestasMap = new HashMap<Long, RespuestaPregunta>();
		for (RespuestaPregunta r : respuestasDB) 
			respuestasMap.put(r.getId(), r);
		
		// para cada una de las respuestas a guardar, ver si existe, si existe actualizar, si no crear una
		for (RespuestaPregunta r : respuestas) {
			
			if (r.getId() != null)  { // si no es neuva
				RespuestaPregunta rDB = respuestasMap.get(r.getId());
				
				if (rDB == null) { // esto es un error, esa pregunta debera existir en la base de datos
					Log.warn("Respuesta con identificador no encontrada en db");
					
					// como la pregunta no existe en db, creamos una nueva
					r.setId(null);
					r.setVersion(null);
				} else {
					// existe, eliminar del map
					respuestasMap.remove(r.getId());
				}
					
			} 
			
			// si no existe en el map, pues nada, se guarda una nueva
			
			// // actualizar la preguntaID, paranoia y guardar la respuesta y actualizar la lista para devolver
			r.setPreguntaId(preguntaId);
			respuestasGuardadads.add ( respuestaDao.save(r));
		}
		
		// guardadas y actualizadas todas las respuestas, lo que queda en el map son repsuestas que han sido 
		// eliminadas y las eliminamos de la base de datos
		for (RespuestaPregunta r : respuestasMap.values()) {
			respuestaDao.remove(r.getId());
		}
		
		return respuestasGuardadads;
	}

	@Override
	public void eliminarRespuestasPregunta(Long preguntaId) {
		List<RespuestaPregunta> respuestasDB = obtenerRespuestasPregunta(preguntaId);

		for (RespuestaPregunta r : respuestasDB) {
			respuestaDao.remove(r.getId());
		}
	}
}
